@@ -1,7 +1,7 @@
 // qtractorMidiControlObserver.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2021, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -36,9 +36,9 @@ static inline float cbrtf2 ( float x )
 	union { float f; int i; } u;
 	u.f  = x;
 	u.i  = (u.i >> 4) + (u.i >> 2);
-	u.i += (u.i >> 4) + 0x2a6497f8;
-	return 0.33333333f * (2.0f * u.f + x / (u.f * u.f));
-//	return u.f;
+	u.i += (u.i >> 4) + 0x2a6a8000; // 0x2a6497f8;
+//	return 0.33333333f * (2.0f * u.f + x / (u.f * u.f));
+	return u.f;
 #else
 	return ::cbrtf(x);
 #endif
