@@ -1,7 +1,7 @@
 // qtractorOptionsForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -264,6 +264,12 @@ qtractorOptionsForm::qtractorOptionsForm ( QWidget *pParent )
 	QObject::connect(m_ui.AudioMetronomeCheckBox,
 		SIGNAL(stateChanged(int)),
 		SLOT(changed()));
+	QObject::connect(m_ui.AudioCountInModeComboBox,
+		SIGNAL(activated(int)),
+		SLOT(changed()));
+	QObject::connect(m_ui.AudioCountInBeatsSpinBox,
+		SIGNAL(valueChanged(int)),
+		SLOT(changed()));
 	QObject::connect(m_ui.MetroBarFilenameComboBox,
 		SIGNAL(editTextChanged(const QString&)),
 		SLOT(changed()));
@@ -327,6 +333,12 @@ qtractorOptionsForm::qtractorOptionsForm ( QWidget *pParent )
 	QObject::connect(m_ui.MidiMetronomeCheckBox,
 		SIGNAL(stateChanged(int)),
 		SLOT(changed()));
+	QObject::connect(m_ui.MidiCountInModeComboBox,
+		SIGNAL(activated(int)),
+		SLOT(changed()));
+	QObject::connect(m_ui.MidiCountInBeatsSpinBox,
+		SIGNAL(valueChanged(int)),
+		SLOT(changed()));
 	QObject::connect(m_ui.MetroChannelSpinBox,
 		SIGNAL(valueChanged(int)),
 		SLOT(updateMetroNoteNames()));
@@ -605,6 +617,8 @@ void qtractorOptionsForm::setOptions ( qtractorOptions *pOptions )
 
 	// Audio metronome options.
 	m_ui.AudioMetronomeCheckBox->setChecked(m_pOptions->bAudioMetronome);
+	m_ui.AudioCountInModeComboBox->setCurrentIndex(m_pOptions->iAudioCountInMode);
+	m_ui.AudioCountInBeatsSpinBox->setValue(m_pOptions->iAudioCountInBeats);
 	m_ui.MetroBarFilenameComboBox->setEditText(m_pOptions->sMetroBarFilename);
 	m_ui.MetroBarGainSpinBox->setValue(log10f2(m_pOptions->fMetroBarGain));
 	m_ui.MetroBeatFilenameComboBox->setEditText(m_pOptions->sMetroBeatFilename);
@@ -638,6 +652,8 @@ void qtractorOptionsForm::setOptions ( qtractorOptions *pOptions )
 
 	// MIDI metronome options.
 	m_ui.MidiMetronomeCheckBox->setChecked(m_pOptions->bMidiMetronome);
+	m_ui.MidiCountInModeComboBox->setCurrentIndex(m_pOptions->iMidiCountInMode);
+	m_ui.MidiCountInBeatsSpinBox->setValue(m_pOptions->iMidiCountInBeats);
 	m_ui.MetroChannelSpinBox->setValue(m_pOptions->iMetroChannel + 1);
 	updateMetroNoteNames();
 	m_ui.MetroBarNoteComboBox->setCurrentIndex(m_pOptions->iMetroBarNote);
@@ -844,6 +860,8 @@ void qtractorOptionsForm::accept (void)
 		m_pOptions->bAudioPlayerAutoConnect = m_ui.AudioPlayerAutoConnectCheckBox->isChecked();
 		// Audio metronome options.
 		m_pOptions->bAudioMetronome      = m_ui.AudioMetronomeCheckBox->isChecked();
+		m_pOptions->iAudioCountInMode    = m_ui.AudioCountInModeComboBox->currentIndex();
+		m_pOptions->iAudioCountInBeats   = m_ui.AudioCountInBeatsSpinBox->value();
 		m_pOptions->sMetroBarFilename    = m_ui.MetroBarFilenameComboBox->currentText();
 		m_pOptions->fMetroBarGain        = pow10f2(m_ui.MetroBarGainSpinBox->value());
 		m_pOptions->sMetroBeatFilename   = m_ui.MetroBeatFilenameComboBox->currentText();
@@ -866,6 +884,8 @@ void qtractorOptionsForm::accept (void)
 		m_pOptions->bMidiControlBus      = m_ui.MidiControlBusCheckBox->isChecked();
 		// MIDI metronome options.
 		m_pOptions->bMidiMetronome       = m_ui.MidiMetronomeCheckBox->isChecked();
+		m_pOptions->iMidiCountInMode     = m_ui.MidiCountInModeComboBox->currentIndex();
+		m_pOptions->iMidiCountInBeats    = m_ui.MidiCountInBeatsSpinBox->value();
 		m_pOptions->iMetroChannel        = m_ui.MetroChannelSpinBox->value() - 1;
 		m_pOptions->iMetroBarNote        = m_ui.MetroBarNoteComboBox->currentIndex();
 		m_pOptions->iMetroBarVelocity    = m_ui.MetroBarVelocitySpinBox->value();
@@ -953,14 +973,29 @@ void qtractorOptionsForm::accept (void)
 		qtractorPluginFactory *pPluginFactory
 			= qtractorPluginFactory::getInstance();
 		if (pPluginFactory) {
-			if (m_iDirtyLadspaPaths > 0 ||
-				m_iDirtyDssiPaths   > 0 ||
-				m_iDirtyVst2Paths   > 0 ||
-				m_iDirtyVst3Paths   > 0 ||
-				m_iDirtyClapPaths   > 0 ||
-				m_iDirtyLv2Paths    > 0) {
-				pPluginFactory->updatePluginPaths();
-				pPluginFactory->clearAll();
+			if (m_iDirtyLadspaPaths > 0) {
+				pPluginFactory->updatePluginPaths(qtractorPluginType::Ladspa);
+				pPluginFactory->clearAll(qtractorPluginType::Ladspa);
+			}
+			if (m_iDirtyDssiPaths > 0){
+				pPluginFactory->updatePluginPaths(qtractorPluginType::Dssi);
+				pPluginFactory->clearAll(qtractorPluginType::Dssi);
+			}
+			if (m_iDirtyVst2Paths > 0){
+				pPluginFactory->updatePluginPaths(qtractorPluginType::Vst2);
+				pPluginFactory->clearAll(qtractorPluginType::Vst2);
+			}
+			if (m_iDirtyVst3Paths > 0){
+				pPluginFactory->updatePluginPaths(qtractorPluginType::Vst3);
+				pPluginFactory->clearAll(qtractorPluginType::Vst3);
+			}
+			if (m_iDirtyClapPaths > 0){
+				pPluginFactory->updatePluginPaths(qtractorPluginType::Clap);
+				pPluginFactory->clearAll(qtractorPluginType::Clap);
+			}
+			if (m_iDirtyLv2Paths > 0){
+				pPluginFactory->updatePluginPaths(qtractorPluginType::Lv2);
+				pPluginFactory->clearAll(qtractorPluginType::Lv2);
 			}
 			if (m_iDirtyBlacklist > 0) {
 				QStringList blacklist;
@@ -1983,6 +2018,10 @@ void qtractorOptionsForm::stabilizeForm (void)
 		m_ui.AudioPlayerBusCheckBox->isChecked());
 
 	const bool bAudioMetronome = m_ui.AudioMetronomeCheckBox->isChecked();
+	m_ui.AudioCountInModeLabel->setEnabled(bAudioMetronome);
+	m_ui.AudioCountInModeComboBox->setEnabled(bAudioMetronome);
+	m_ui.AudioCountInBeatsSpinBox->setEnabled(
+		bAudioMetronome && m_ui.AudioCountInModeComboBox->currentIndex() > 0);
 	m_ui.MetroBarFilenameTextLabel->setEnabled(bAudioMetronome);
 	m_ui.MetroBarFilenameComboBox->setEnabled(bAudioMetronome);
 	m_ui.MetroBarFilenameToolButton->setEnabled(bAudioMetronome);
@@ -2005,6 +2044,10 @@ void qtractorOptionsForm::stabilizeForm (void)
 	m_ui.MidiMmcDeviceComboBox->setEnabled(bMmcMode);
 
 	const bool bMidiMetronome = m_ui.MidiMetronomeCheckBox->isChecked();
+	m_ui.MidiCountInModeLabel->setEnabled(bMidiMetronome);
+	m_ui.MidiCountInModeComboBox->setEnabled(bMidiMetronome);
+	m_ui.MidiCountInBeatsSpinBox->setEnabled(
+		bMidiMetronome && m_ui.MidiCountInModeComboBox->currentIndex() > 0);
 	m_ui.MetroChannelTextLabel->setEnabled(bMidiMetronome);
 	m_ui.MetroChannelSpinBox->setEnabled(bMidiMetronome);
 	m_ui.MetroBarNoteTextLabel->setEnabled(bMidiMetronome);
