@@ -1,7 +1,7 @@
 // qtractorVst2Plugin.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -1581,7 +1581,7 @@ static VstIntPtr qtractorVst2Plugin_openFileSelector (
 		if (pParentWidget)
 			pParentWidget = pParentWidget->window();
 		const QString& sTitle = QString("%1 - %2")
-			.arg(pvfs->title).arg((pVst2Plugin->type())->name());
+			.arg(pvfs->title).arg(pVst2Plugin->title());
 		const QString& sDirectory = pvfs->initialPath;
 		const QString& sFilter = filters.join(";;");
 		QFileDialog::Options options;
@@ -1610,7 +1610,7 @@ static VstIntPtr qtractorVst2Plugin_openFileSelector (
 		if (pParentWidget)
 			pParentWidget = pParentWidget->window();
 		const QString& sTitle = QString("%1 - %2")
-			.arg(pvfs->title).arg((pVst2Plugin->type())->name());
+			.arg(pvfs->title).arg(pVst2Plugin->title());
 		const QFileDialog::Options options
 			= QFileDialog::ShowDirsOnly | QFileDialog::DontUseNativeDialog;
 		const QString& sDirectory
@@ -1785,7 +1785,7 @@ static VstIntPtr VSTCALLBACK qtractorVst2Plugin_HostCallback ( AEffect *effect,
 			if (pSession) {
 				qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
 				if (pAudioEngine)
-					ret = (VstIntPtr) pAudioEngine->bufferSize();
+					ret = (VstIntPtr) pAudioEngine->blockSize();
 			}
 		}
 		break;
