@@ -1,7 +1,7 @@
 // qtractorVst3Plugin.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -1962,6 +1962,16 @@ public:
 	{
 		m_runLoop = owned(NEW RunLoop());
 		m_plugView->setFrame(this);
+
+		ViewRect rect;
+		if (m_plugView->getSize(&rect) == kResultOk) {
+			m_resizing = true;
+			const QSize size(
+				rect.right  - rect.left,
+				rect.bottom - rect.top);
+			m_widget->resize(size);
+			m_resizing = false;
+		}
 	}
 
 	// Destructor.
@@ -1984,7 +1994,9 @@ public:
 		if (!rect || !plugView || plugView != m_plugView)
 			return kInvalidArgument;
 
-		if (!m_widget || m_resizing)
+		if (!m_widget)
+			return kInternalError;
+		if (m_resizing)
 			return kResultFalse;
 
 		m_resizing = true;
@@ -1995,10 +2007,10 @@ public:
 		qDebug("qtractorVst3Plugin::EditorFrame[%p]::resizeView(%p, %p) size=(%d, %d)",
 			this, plugView, rect, size.width(), size.height());
 	#endif
-		if (m_plugView->canResize() != kResultOk)
-			m_widget->setFixedSize(size);
-		else
+		if (m_plugView->canResize() == kResultOk)
 			m_widget->resize(size);
+		else
+			m_widget->setFixedSize(size);
 		m_resizing = false;
 
 		ViewRect rect0;
@@ -2009,7 +2021,7 @@ public:
 			rect0.right  - rect0.left,
 			rect0.bottom - rect0.top);
 		if (size != size0)
-			m_plugView->onSize(rect);
+			m_plugView->onSize(&rect0);
 
 		return kResultOk;
 	}
@@ -2425,16 +2437,12 @@ bool qtractorVst3Plugin::Impl::process_reset (
 	m_events_in.clear();
 	m_events_out.clear();
 
-	const bool         bFreewheel  = pAudioEngine->isFreewheel();
-	const unsigned int iSampleRate = pAudioEngine->sampleRate();
-	const unsigned int iBufferSize = pAudioEngine->bufferSize();
-	const unsigned int iBufferSizeEx = pAudioEngine->bufferSizeEx();
-
 	Vst::ProcessSetup setup;
-	setup.processMode        = (bFreewheel ? Vst::kOffline :  Vst::kRealtime);
+	const bool bFreewheel    = pAudioEngine->isFreewheel();
+	setup.processMode        = (bFreewheel ? Vst::kOffline :Vst::kRealtime);
 	setup.symbolicSampleSize = Vst::kSample32;
-	setup.maxSamplesPerBlock = iBufferSizeEx;
-	setup.sampleRate         = float(iSampleRate);
+	setup.maxSamplesPerBlock = pAudioEngine->bufferSizeEx();
+	setup.sampleRate         = float(pAudioEngine->sampleRate());
 
 	if (m_processor->setupProcessing(setup) != kResultOk)
 		return false;
@@ -2449,7 +2457,7 @@ bool qtractorVst3Plugin::Impl::process_reset (
 	m_buffers_out.channelBuffers32 = nullptr;
 
 	// Setup processor data struct...
-	m_process_data.numSamples             = iBufferSize;
+	m_process_data.numSamples             = pAudioEngine->blockSize();
 	m_process_data.symbolicSampleSize     = Vst::kSample32;
 
 	if (pType->audioIns() > 0) {
@@ -2884,17 +2892,17 @@ void qtractorVst3Plugin::EditorWidget::resizeEvent ( QResizeEvent *pResizeEvent
 		rect.top    = 0;
 		rect.right  = size.width();
 		rect.bottom = size.height();
-		if (plugView->checkSizeConstraint(&rect) == kResultOk) {
-			const QSize size2(
-				rect.right - rect.left,
-				rect.bottom - rect.top);
-			if (size2 != size) {
-				m_resizing = true;
-				QWidget::resize(size2);
-				m_resizing = false;
-			}
+		if (plugView->checkSizeConstraint(&rect) != kResultOk)
+			plugView->getSize(&rect);
+		const QSize size2(
+			rect.right - rect.left,
+			rect.bottom - rect.top);
+		if (size2 != size) {
+			m_resizing = true;
+			QWidget::resize(size2);
+			m_resizing = false;
+			plugView->onSize(&rect);
 		}
-		plugView->onSize(&rect);
 	}
 }
 
@@ -3317,10 +3325,6 @@ void qtractorVst3Plugin::openEditor ( QWidget *pParent )
 
 	m_pEditorFrame = new EditorFrame(plugView, m_pEditorWidget);
 
-	ViewRect rect;
-	if (plugView->getSize(&rect) == kResultOk)
-		m_pEditorFrame->resizeView(plugView, &rect);
-
 	void *wid = (void *) m_pEditorWidget->parentWinId();
 	if (plugView->attached(wid, kPlatformTypeX11EmbedWindowID) != kResultOk) {
 	#ifdef CONFIG_DEBUG
